@@ -1,4 +1,11 @@
-/* $Id: landscape.cpp 15903 2009-03-30 23:15:05Z rubidium $ */
+/* $Id: landscape.cpp 19665 2010-04-17 22:27:49Z rubidium $ */
+
+/*
+ * This file is part of OpenTTD.
+ * OpenTTD is free software; you can redistribute it and/or modify it under the terms of the GNU General Public License as published by the Free Software Foundation, version 2.
+ * OpenTTD is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
+ * See the GNU General Public License for more details. You should have received a copy of the GNU General Public License along with OpenTTD. If not, see <http://www.gnu.org/licenses/>.
+ */
 
 /** @file landscape.cpp Functions related to the landscape (slopes etc.). */
 
@@ -21,7 +28,8 @@
 #include "water.h"
 #include "effectvehicle_func.h"
 #include "landscape_type.h"
-#include "settings_type.h"
+#include "animated_tile_func.h"
+#include "core/random_func.hpp"
 
 #include "table/sprites.h"
 
@@ -68,7 +76,7 @@ const byte _tileh_to_sprite[32] = {
  * Otherwise it points to a table loaded from a newGRF file, that describes the variable snowline
  * @ingroup SnowLineGroup
  * @see GetSnowLine() GameCreationSettings */
-SnowLine *_snow_line = NULL;
+static SnowLine *_snow_line = NULL;
 
 /**
  * Applies a foundation to a slope.
@@ -331,7 +339,7 @@ Slope GetFoundationSlope(TileIndex tile, uint *z)
 }
 
 
-static bool HasFoundationNW(TileIndex tile, Slope slope_here, uint z_here)
+bool HasFoundationNW(TileIndex tile, Slope slope_here, uint z_here)
 {
 	uint z;
 
@@ -348,7 +356,7 @@ static bool HasFoundationNW(TileIndex tile, Slope slope_here, uint z_here)
 }
 
 
-static bool HasFoundationNE(TileIndex tile, Slope slope_here, uint z_here)
+bool HasFoundationNE(TileIndex tile, Slope slope_here, uint z_here)
 {
 	uint z;
 
@@ -471,6 +479,9 @@ void DrawFoundation(TileInfo *ti, Foundation f)
 
 void DoClearSquare(TileIndex tile)
 {
+	/* If the tile can have animation and we clear it, delete it from the animated tile list. */
+	if (_tile_type_procs[GetTileType(tile)]->animate_tile_proc != NULL) DeleteAnimatedTile(tile);
+
 	MakeClear(tile, CLEAR_GRASS, _generating_world ? 3 : 0);
 	MarkTileDirtyByTile(tile);
 }
@@ -500,22 +511,6 @@ void ChangeTileOwner(TileIndex tile, Owner old_owner, Owner new_owner)
 	_tile_type_procs[GetTileType(tile)]->change_tile_owner_proc(tile, old_owner, new_owner);
 }
 
-void GetAcceptedCargo(TileIndex tile, AcceptedCargo ac)
-{
-	memset(ac, 0, sizeof(AcceptedCargo));
-	_tile_type_procs[GetTileType(tile)]->get_accepted_cargo_proc(tile, ac);
-}
-
-void AnimateTile(TileIndex tile)
-{
-	_tile_type_procs[GetTileType(tile)]->animate_tile_proc(tile);
-}
-
-bool ClickTile(TileIndex tile)
-{
-	return _tile_type_procs[GetTileType(tile)]->click_tile_proc(tile);
-}
-
 void GetTileDesc(TileIndex tile, TileDesc *td)
 {
 	_tile_type_procs[GetTileType(tile)]->get_tile_desc_proc(tile, td);
@@ -526,7 +521,7 @@ void GetTileDesc(TileIndex tile, TileDesc *td)
  * @return true if the table has been loaded already.
  * @ingroup SnowLineGroup
  */
-bool IsSnowLineSet(void)
+bool IsSnowLineSet()
 {
 	return _snow_line != NULL;
 }
@@ -555,7 +550,7 @@ void SetSnowLine(byte table[SNOW_LINE_MONTHS][SNOW_LINE_DAYS])
  * @return the snow line height.
  * @ingroup SnowLineGroup
  */
-byte GetSnowLine(void)
+byte GetSnowLine()
 {
 	if (_snow_line == NULL) return _settings_game.game_creation.snow_line;
 
@@ -569,7 +564,7 @@ byte GetSnowLine(void)
  * @return the highest snow line height.
  * @ingroup SnowLineGroup
  */
-byte HighestSnowLine(void)
+byte HighestSnowLine()
 {
 	return _snow_line == NULL ? _settings_game.game_creation.snow_line : _snow_line->highest_value;
 }
@@ -579,7 +574,7 @@ byte HighestSnowLine(void)
  * @return the lowest snow line height.
  * @ingroup SnowLineGroup
  */
-byte LowestSnowLine(void)
+byte LowestSnowLine()
 {
 	return _snow_line == NULL ? _settings_game.game_creation.snow_line : _snow_line->lowest_value;
 }
@@ -588,7 +583,7 @@ byte LowestSnowLine(void)
  * Clear the variable snow line table and free the memory.
  * @ingroup SnowLineGroup
  */
-void ClearSnowLine(void)
+void ClearSnowLine()
 {
 	free(_snow_line);
 	_snow_line = NULL;
@@ -599,6 +594,8 @@ void ClearSnowLine(void)
  * @param flags of operation to conduct
  * @param p1 unused
  * @param p2 unused
+ * @param text unused
+ * @return the cost of this operation or an error
  */
 CommandCost CmdLandscapeClear(TileIndex tile, DoCommandFlag flags, uint32 p1, uint32 p2, const char *text)
 {
@@ -607,9 +604,11 @@ CommandCost CmdLandscapeClear(TileIndex tile, DoCommandFlag flags, uint32 p1, ui
 
 /** Clear a big piece of landscape
  * @param tile end tile of area dragging
- * @param p1 start tile of area dragging
  * @param flags of operation to conduct
+ * @param p1 start tile of area dragging
  * @param p2 unused
+ * @param text unused
+ * @return the cost of this operation or an error
  */
 CommandCost CmdClearArea(TileIndex tile, DoCommandFlag flags, uint32 p1, uint32 p2, const char *text)
 {
@@ -630,7 +629,7 @@ CommandCost CmdClearArea(TileIndex tile, DoCommandFlag flags, uint32 p1, uint32
 	for (int x = sx; x <= ex; ++x) {
 		for (int y = sy; y <= ey; ++y) {
 			CommandCost ret = DoCommand(TileXY(x, y), 0, 0, flags & ~DC_EXEC, CMD_LANDSCAPE_CLEAR);
-			if (CmdFailed(ret)) continue;
+			if (ret.Failed()) continue;
 			success = true;
 
 			if (flags & DC_EXEC) {
@@ -858,78 +857,79 @@ static void CreateDesertOrRainForest()
 
 void GenerateLandscape(byte mode)
 {
-	static const int gwp_desert_amount = 4 + 8;
+	/** Number of steps of landscape generation */
+	enum GenLandscapeSteps {
+		GLS_HEIGHTMAP    =  3, ///< Loading a heightmap
+		GLS_TERRAGENESIS =  5, ///< Terragenesis generator
+		GLS_ORIGINAL     =  2, ///< Original generator
+		GLS_TROPIC       = 12, ///< Extra steps needed for tropic landscape
+		GLS_OTHER        =  0, ///< Extra steps for other landscapes
+	};
+	uint steps = (_settings_game.game_creation.landscape == LT_TROPIC) ? GLS_TROPIC : GLS_OTHER;
 
-	if (mode == GW_HEIGHTMAP) {
-		SetGeneratingWorldProgress(GWP_LANDSCAPE, (_settings_game.game_creation.landscape == LT_TROPIC) ? 1 + gwp_desert_amount : 1);
+	if (mode == GWM_HEIGHTMAP) {
+		SetGeneratingWorldProgress(GWP_LANDSCAPE, steps + GLS_HEIGHTMAP);
 		LoadHeightmap(_file_to_saveload.name);
 		IncreaseGeneratingWorldProgress(GWP_LANDSCAPE);
 	} else if (_settings_game.game_creation.land_generator == LG_TERRAGENESIS) {
-		SetGeneratingWorldProgress(GWP_LANDSCAPE, (_settings_game.game_creation.landscape == LT_TROPIC) ? 3 + gwp_desert_amount : 3);
+		SetGeneratingWorldProgress(GWP_LANDSCAPE, steps + GLS_TERRAGENESIS);
 		GenerateTerrainPerlin();
 	} else {
+		SetGeneratingWorldProgress(GWP_LANDSCAPE, steps + GLS_ORIGINAL);
 		if (_settings_game.construction.freeform_edges) {
 			for (uint x = 0; x < MapSizeX(); x++) MakeVoid(TileXY(x, 0));
 			for (uint y = 0; y < MapSizeY(); y++) MakeVoid(TileXY(0, y));
 		}
 		switch (_settings_game.game_creation.landscape) {
 			case LT_ARCTIC: {
-				SetGeneratingWorldProgress(GWP_LANDSCAPE, 2);
-
 				uint32 r = Random();
 
 				for (uint i = ScaleByMapSize(GB(r, 0, 7) + 950); i != 0; --i) {
 					GenerateTerrain(2, 0);
 				}
-				IncreaseGeneratingWorldProgress(GWP_LANDSCAPE);
 
 				uint flag = GB(r, 7, 2) | 4;
 				for (uint i = ScaleByMapSize(GB(r, 9, 7) + 450); i != 0; --i) {
 					GenerateTerrain(4, flag);
 				}
-				IncreaseGeneratingWorldProgress(GWP_LANDSCAPE);
 			} break;
 
 			case LT_TROPIC: {
-				SetGeneratingWorldProgress(GWP_LANDSCAPE, 3 + gwp_desert_amount);
-
 				uint32 r = Random();
 
 				for (uint i = ScaleByMapSize(GB(r, 0, 7) + 170); i != 0; --i) {
 					GenerateTerrain(0, 0);
 				}
-				IncreaseGeneratingWorldProgress(GWP_LANDSCAPE);
 
 				uint flag = GB(r, 7, 2) | 4;
 				for (uint i = ScaleByMapSize(GB(r, 9, 8) + 1700); i != 0; --i) {
 					GenerateTerrain(0, flag);
 				}
-				IncreaseGeneratingWorldProgress(GWP_LANDSCAPE);
 
 				flag ^= 2;
 
 				for (uint i = ScaleByMapSize(GB(r, 17, 7) + 410); i != 0; --i) {
 					GenerateTerrain(3, flag);
 				}
-				IncreaseGeneratingWorldProgress(GWP_LANDSCAPE);
 			} break;
 
 			default: {
-				SetGeneratingWorldProgress(GWP_LANDSCAPE, 1);
-
 				uint32 r = Random();
 
 				uint i = ScaleByMapSize(GB(r, 0, 7) + (3 - _settings_game.difficulty.quantity_sea_lakes) * 256 + 100);
 				for (; i != 0; --i) {
 					GenerateTerrain(_settings_game.difficulty.terrain_type, 0);
 				}
-				IncreaseGeneratingWorldProgress(GWP_LANDSCAPE);
 			} break;
 		}
 	}
 
+	/* Do not call IncreaseGeneratingWorldProgress() before FixSlopes(),
+	 * it allows screen redraw. Drawing of broken slopes crashes the game */
 	FixSlopes();
+	IncreaseGeneratingWorldProgress(GWP_LANDSCAPE);
 	ConvertGroundTilesIntoWaterTiles();
+	IncreaseGeneratingWorldProgress(GWP_LANDSCAPE);
 
 	if (_settings_game.game_creation.landscape == LT_TROPIC) CreateDesertOrRainForest();
 }
@@ -940,7 +940,6 @@ void OnTick_Station();
 void OnTick_Industry();
 
 void OnTick_Companies();
-void OnTick_Train();
 
 void CallLandscapeTick()
 {
@@ -950,5 +949,4 @@ void CallLandscapeTick()
 	OnTick_Industry();
 
 	OnTick_Companies();
-	OnTick_Train();
 }
