@@ -1,4 +1,4 @@
-/* $Id: newgrf_spritegroup.h 19056 2010-02-07 22:22:54Z peter1138 $ */
+/* $Id: newgrf_spritegroup.h 19551 2010-04-03 19:48:01Z rubidium $ */
 
 /*
  * This file is part of OpenTTD.
@@ -31,7 +31,7 @@
  */
 static inline uint32 GetRegister(uint i)
 {
-	extern TemporaryStorageArray<uint32, 0x110> _temp_store;
+	extern TemporaryStorageArray<int32, 0x110> _temp_store;
 	return _temp_store.Get(i);
 }
 
